@@ -35,6 +35,7 @@
 #include <gconf/gconf-client.h>
 
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-help.h> /* for gnome_help_display */
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -53,10 +54,6 @@
 #define KEY_ACTIVATE_DELAY  KEY_DIR "/idle_delay"
 #define KEY_LOCK_DELAY      KEY_DIR "/lock_delay"
 #define KEY_CYCLE_DELAY     KEY_DIR "/cycle_delay"
-#define KEY_DPMS_ENABLED    KEY_DIR "/dpms_enabled"
-#define KEY_DPMS_STANDBY    KEY_DIR "/dpms_standby"
-#define KEY_DPMS_SUSPEND    KEY_DIR "/dpms_suspend"
-#define KEY_DPMS_OFF        KEY_DIR "/dpms_off"
 #define KEY_THEMES          KEY_DIR "/themes"
 
 enum {
@@ -392,8 +389,21 @@ response_cb (GtkWidget *widget,
              int        response_id)
 {
 
-        if (response_id == GTK_RESPONSE_HELP)
+        if (response_id == GTK_RESPONSE_HELP) {
+                GError *error;
+                error = NULL;
+                gnome_help_display_desktop (NULL,
+                                            "user-guide",
+                                            "user-guide.xml",
+                                            "prefs-screensaver",
+                                            &error);
+                if (error != NULL) {
+                        g_warning ("%s", error->message);
+                        g_error_free (error);
+                }
+
                 return;
+        }
 
         gtk_widget_destroy (widget);
         gtk_main_quit ();
@@ -611,8 +621,9 @@ setup_treeview_selection (GtkWidget *tree)
 
         theme = config_get_theme (&is_writable);
 
-        if (! is_writable)
+        if (! is_writable) {
                 gtk_widget_set_sensitive (tree, FALSE);
+        }
 
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree));
 
@@ -955,8 +966,9 @@ key_changed_cb (GConfClient *client,
 
         key = gconf_entry_get_key (entry);
 
-        if (! g_str_has_prefix (key, KEY_DIR))
+        if (! g_str_has_prefix (key, KEY_DIR)) {
                 return;
+        }
 
         value = gconf_entry_get_value (entry);
 
@@ -980,6 +992,15 @@ key_changed_cb (GConfClient *client,
                 } else {
                         invalid_type_warning (key);
                 }
+        } else if (strcmp (key, KEY_THEMES) == 0) {
+                if (value->type == GCONF_VALUE_LIST) {
+                        GtkWidget *treeview;
+
+                        treeview = glade_xml_get_widget (xml, "savers_treeview");
+                        setup_treeview_selection (treeview);
+                } else {
+                        invalid_type_warning (key);
+                }
         } else if (strcmp (key, KEY_ACTIVATE_DELAY) == 0) {
 
                 if (value->type == GCONF_VALUE_INT) {
@@ -1139,8 +1160,11 @@ main (int    argc,
         textdomain (GETTEXT_PACKAGE);
 #endif 
 
-        gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
-                            GNOME_PARAM_NONE);
+        gnome_program_init (PACKAGE, VERSION,
+                            LIBGNOMEUI_MODULE,
+                            argc, argv,
+                            GNOME_PROGRAM_STANDARD_PROPERTIES,
+                            NULL);
 
         job = gs_job_new ();
 
