@@ -54,9 +54,11 @@ static DBusHandlerResult gs_listener_message_handler    (DBusConnection  *connec
                                                          DBusMessage     *message,
                                                          void            *user_data);
 
-#define GS_LISTENER_SERVICE   "org.gnome.screensaver"
-#define GS_LISTENER_PATH      "/org/gnome/screensaver"
-#define GS_LISTENER_INTERFACE "org.gnome.screensaver"
+#define GS_LISTENER_SERVICE   "org.gnome.ScreenSaver"
+#define GS_LISTENER_PATH      "/org/gnome/ScreenSaver"
+#define GS_LISTENER_INTERFACE "org.gnome.ScreenSaver"
+
+#define TYPE_MISMATCH_ERROR GS_LISTENER_INTERFACE ".TypeMismatch"
 
 #define GS_LISTENER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_LISTENER, GSListenerPrivate))
 
@@ -64,12 +66,14 @@ struct GSListenerPrivate
 {
         DBusConnection *connection;
 
-        guint           idle : 1;
+        guint           session_idle : 1;
         guint           active : 1;
+        guint           activation_enabled : 1;
         guint           throttle_enabled : 1;
         GHashTable     *inhibitors;
 
         time_t          active_start;
+        time_t          session_idle_start;
 };
 
 enum {
@@ -85,8 +89,9 @@ enum {
 enum {
         PROP_0,
         PROP_ACTIVE,
-        PROP_IDLE,
+        PROP_SESSION_IDLE,
         PROP_THROTTLE_ENABLED,
+        PROP_ACTIVATION_ENABLED,
 };
 
 
@@ -168,6 +173,30 @@ gs_listener_send_signal_active_changed (GSListener *listener)
         dbus_message_unref (message);
 }
 
+static void
+gs_listener_send_signal_session_idle_changed (GSListener *listener)
+{
+        DBusMessage    *message;
+        DBusMessageIter iter;
+        dbus_bool_t     idle;
+
+        g_return_if_fail (listener != NULL);
+
+        message = dbus_message_new_signal (GS_LISTENER_PATH,
+                                           GS_LISTENER_SERVICE,
+                                           "SessionIdleChanged");
+
+        idle = listener->priv->session_idle;
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &idle);
+
+        if (! send_dbus_message (listener->priv->connection, message)) {
+                g_warning ("Could not send SessionIdleChanged signal");
+        }
+
+        dbus_message_unref (message);
+}
+
 static void
 gs_listener_send_signal_throttle_enabled_changed (GSListener *listener)
 {
@@ -201,10 +230,10 @@ list_inhibitors (gpointer key,
 }
 
 static gboolean
-listener_check_activation (GSListener *listener)
+listener_is_inhibited (GSListener *listener)
 {
         guint    n_inhibitors;
-        gboolean res;
+        gboolean inhibited;
 
         /* if we aren't inhibited then activate */
         n_inhibitors = 0;
@@ -214,14 +243,78 @@ listener_check_activation (GSListener *listener)
                 g_hash_table_foreach (listener->priv->inhibitors, list_inhibitors, NULL);
         }
 
+        inhibited = (n_inhibitors > 0);
+
+        return inhibited;
+}
+
+static gboolean
+listener_check_activation (GSListener *listener)
+{
+        gboolean inhibited;
+        gboolean res;
+
+        gs_debug ("Checking for activation");
+
+        if (! listener->priv->activation_enabled) {
+                return TRUE;
+        }
+
+        if (! listener->priv->session_idle) {
+                return TRUE;
+        }
+
+        /* if we aren't inhibited then activate */
+        inhibited = listener_is_inhibited (listener);
+
         res = FALSE;
-        if (listener->priv->idle && n_inhibitors == 0) {
+        if (! inhibited) {
+                gs_debug ("Trying to activate");
                 res = gs_listener_set_active (listener, TRUE);
         }
 
         return res;
 }
 
+static gboolean
+listener_set_session_idle_internal (GSListener *listener,
+                                    gboolean    idle)
+{
+        listener->priv->session_idle = idle;
+
+        if (idle) {
+                listener->priv->session_idle_start = time (NULL);
+        } else {
+                listener->priv->session_idle_start = 0;
+        }
+
+        gs_listener_send_signal_session_idle_changed (listener);
+
+        return TRUE;
+}
+
+static gboolean
+listener_set_active_internal (GSListener *listener,
+                              gboolean    active)
+{
+        listener->priv->active = active;
+
+        /* if idle not in sync with active, change it */
+        if (listener->priv->session_idle != active) {
+                listener_set_session_idle_internal (listener, active);
+        }
+
+        if (active) {
+                listener->priv->active_start = time (NULL);
+        } else {
+                listener->priv->active_start = 0;
+        }
+
+        gs_listener_send_signal_active_changed (listener);
+
+        return TRUE;
+}
+
 gboolean
 gs_listener_set_active (GSListener *listener,
                         gboolean    active)
@@ -230,70 +323,75 @@ gs_listener_set_active (GSListener *listener,
 
         g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
 
-        if (listener->priv->active != active) {
+        if (listener->priv->active == active) {
+                gs_debug ("Trying to set active state when already: %s",
+                          active ? "active" : "inactive");
+                return FALSE;
+        }
 
-                res = FALSE;
-                g_signal_emit (listener, signals [ACTIVE_CHANGED], 0, active, &res);
-                if (! res) {
-                        /* if the signal is not handled then we haven't changed state */
-
-                        /* clear the idle state */
-                        if (active) {
-                                gs_listener_set_idle (listener, FALSE);
-                        }
-
-                        return FALSE;
-                }
-
-                listener->priv->active = active;
-                listener->priv->idle = active;
+        res = FALSE;
+        g_signal_emit (listener, signals [ACTIVE_CHANGED], 0, active, &res);
+        if (! res) {
+                /* if the signal is not handled then we haven't changed state */
+                gs_debug ("Active-changed signal not handled");
 
+                /* clear the idle state */
                 if (active) {
-                        listener->priv->active_start = time (NULL);
-                } else {
-                        listener->priv->active_start = 0;
+                        listener_set_session_idle_internal (listener, FALSE);
                 }
 
-                gs_listener_send_signal_active_changed (listener);
+                return FALSE;
+        }
 
-                if (! active) {
-                        /* if we are deactivating then reset the throttle */
-                        gs_listener_set_throttle_enabled (listener, FALSE);
-                }
+        listener_set_active_internal (listener, active);
+
+        if (! active) {
+                /* if we are deactivating then reset the throttle */
+                gs_listener_set_throttle_enabled (listener, FALSE);
         }
 
         return TRUE;
 }
 
 gboolean
-gs_listener_set_idle (GSListener *listener,
-                      gboolean    idle)
+gs_listener_set_session_idle (GSListener *listener,
+                              gboolean    idle)
 {
+        gboolean res;
+
         g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
 
-        if (listener->priv->idle == idle) {
-                g_warning ("Trying to set idle when already idle");
+        gs_debug ("Setting session idle: %d", idle);
+
+        if (listener->priv->session_idle == idle) {
+                gs_debug ("Trying to set idle state when already %s",
+                          idle ? "idle" : "not idle");
                 return FALSE;
         }
 
         if (idle) {
-                guint n_inhibitors = 0;
+                gboolean inhibited;
 
-                /* if we aren't inhibited then set idle */
-                if (listener->priv->inhibitors) {
-                        n_inhibitors = g_hash_table_size (listener->priv->inhibitors);
-                }
+                inhibited = listener_is_inhibited (listener);
 
-                if (n_inhibitors != 0) {
+                /* if we are inhibited then do nothing */
+                if (inhibited) {
                         return FALSE;
                 }
         }
 
-        listener->priv->idle = idle;
+        listener->priv->session_idle = idle;
+        res = listener_check_activation (listener);
 
-        listener_check_activation (listener);
+        /* if activation fails then don't set idle */
+        if (res) {
+                listener_set_session_idle_internal (listener, idle);
+        } else {
+                gs_debug ("Idle activation failed");
+                listener->priv->session_idle = !idle;
+        }
 
-        return TRUE;
+        return res;
 }
 
 void
@@ -311,6 +409,37 @@ gs_listener_set_throttle_enabled (GSListener *listener,
         }
 }
 
+gboolean
+gs_listener_get_activation_enabled (GSListener *listener)
+{
+        g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
+
+        return listener->priv->activation_enabled;
+}
+
+gboolean
+gs_listener_is_inhibited (GSListener *listener)
+{
+        gboolean inhibited;
+
+        g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
+
+        inhibited = listener_is_inhibited (listener);
+
+        return inhibited;
+}
+
+void
+gs_listener_set_activation_enabled (GSListener *listener,
+                                    gboolean    enabled)
+{
+        g_return_if_fail (GS_IS_LISTENER (listener));
+
+        if (listener->priv->activation_enabled != enabled) {
+                listener->priv->activation_enabled = enabled;
+        }
+}
+
 static dbus_bool_t
 listener_property_set_bool (GSListener *listener,
                             guint       prop_id,
@@ -325,10 +454,6 @@ listener_property_set_bool (GSListener *listener,
                 gs_listener_set_active (listener, value);
                 ret = TRUE;
                 break;
-        case PROP_IDLE:
-                gs_listener_set_idle (listener, value);
-                ret = TRUE;
-                break;
         case PROP_THROTTLE_ENABLED:
                 gs_listener_set_throttle_enabled (listener, value);
                 ret = TRUE;
@@ -356,10 +481,12 @@ raise_error (DBusConnection *connection,
 
         g_warning (buf);
         reply = dbus_message_new_error (in_reply_to, error_name, buf);
-        if (reply == NULL)
+        if (reply == NULL) {
                 g_error ("No memory");
-        if (! dbus_connection_send (connection, reply, NULL))
+        }
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 }
@@ -498,12 +625,14 @@ raise_property_type_error (DBusConnection *connection,
         g_warning (buf);
 
         reply = dbus_message_new_error (in_reply_to,
-                                        "org.gnome.screensaver.TypeMismatch",
+                                        TYPE_MISMATCH_ERROR,
                                         buf);
-        if (reply == NULL)
+        if (reply == NULL) {
                 g_error ("No memory");
-        if (! dbus_connection_send (connection, reply, NULL))
+        }
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 }
@@ -546,11 +675,13 @@ listener_set_property (GSListener     *listener,
 
         reply = dbus_message_new_method_return (message);
 
-        if (reply == NULL)
+        if (reply == NULL) {
                 g_error ("No memory");
+        }
 
-        if (! dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 
@@ -581,10 +712,10 @@ listener_get_property (GSListener     *listener,
                         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &b);
                 }
                 break;
-        case PROP_IDLE:
+        case PROP_SESSION_IDLE:
                 {
                         dbus_bool_t b;
-                        b = listener->priv->idle;
+                        b = listener->priv->session_idle;
                         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &b);
                 }
                 break;
@@ -600,8 +731,9 @@ listener_get_property (GSListener     *listener,
                 break;
         }
 
-        if (! dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 
@@ -655,6 +787,53 @@ listener_get_active_time (GSListener     *listener,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+listener_get_session_idle_time (GSListener     *listener,
+                                DBusConnection *connection,
+                                DBusMessage    *message)
+{
+        DBusMessageIter iter;
+        DBusMessage    *reply;
+        dbus_uint32_t    secs;
+
+        reply = dbus_message_new_method_return (message);
+
+        dbus_message_iter_init_append (reply, &iter);
+
+        if (reply == NULL) {
+                g_error ("No memory");
+        }
+
+        if (listener->priv->session_idle) {
+                time_t now = time (NULL);
+
+                if (now < listener->priv->session_idle_start) {
+                        /* shouldn't happen */
+                        g_warning ("Session idle start time is in the future");
+                        secs = 0;
+                } else if (listener->priv->session_idle_start <= 0) {
+                        /* shouldn't happen */
+                        g_warning ("Session idle start time was not set");
+                        secs = 0;
+                } else {
+                        secs = now - listener->priv->session_idle_start;
+                }
+        } else {
+                secs = 0;
+        }
+
+        gs_debug ("Returning session idle for %u seconds", secs);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &secs);
+
+        if (! dbus_connection_send (connection, reply, NULL)) {
+                g_error ("No memory");
+        }
+
+        dbus_message_unref (reply);
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 listener_dbus_filter_handle_methods (DBusConnection *connection,
                                      DBusMessage    *message, 
@@ -674,15 +853,15 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
         g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
         g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
 
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "lock")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "Lock")) {
                 g_signal_emit (listener, signals [LOCK], 0);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "quit")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "Quit")) {
                 g_signal_emit (listener, signals [QUIT], 0);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "cycle")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "Cycle")) {
                 g_signal_emit (listener, signals [CYCLE], 0);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
@@ -701,16 +880,23 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getActiveTime")) {
                 return listener_get_active_time (listener, connection, message);
         }
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getSessionIdle")) {
+                return listener_get_property (listener, connection, message, PROP_SESSION_IDLE);
+        }
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getSessionIdleTime")) {
+                return listener_get_session_idle_time (listener, connection, message);
+        }
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "setThrottleEnabled")) {
                 return listener_set_property (listener, connection, message, PROP_THROTTLE_ENABLED);
         }
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getThrottleEnabled")) {
                 return listener_get_property (listener, connection, message, PROP_THROTTLE_ENABLED);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "poke")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "Poke")) {
                 g_signal_emit (listener, signals [POKE], 0);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
+
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
@@ -735,11 +921,13 @@ gs_listener_message_handler (DBusConnection *connection,
 
                 reply = dbus_message_new_method_return (message);
 
-                if (reply == NULL)
+                if (reply == NULL) {
                         g_error ("No memory");
+                }
 
-                if (! dbus_connection_send (connection, reply, NULL))
+                if (! dbus_connection_send (connection, reply, NULL)) {
                         g_error ("No memory");
+                }
 
                 dbus_message_unref (reply);
 
@@ -750,8 +938,9 @@ gs_listener_message_handler (DBusConnection *connection,
                 dbus_connection_unref (connection);
 
                 return DBUS_HANDLER_RESULT_HANDLED;
-        } 
-        else return listener_dbus_filter_handle_methods (connection, message, user_data, TRUE);
+        } else {
+                return listener_dbus_filter_handle_methods (connection, message, user_data, TRUE);
+        }
 }
 
 static gboolean
@@ -829,8 +1018,9 @@ listener_dbus_filter_function (DBusConnection *connection,
 
                 if (listener->priv->inhibitors != NULL)
                         listener_service_deleted (listener, message);
-        } else 
+        } else {
                 return listener_dbus_filter_handle_methods (connection, message, user_data, FALSE);
+        }
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -849,12 +1039,15 @@ gs_listener_set_property (GObject            *object,
         case PROP_ACTIVE:
                 gs_listener_set_active (self, g_value_get_boolean (value));
                 break;
-        case PROP_IDLE:
-                gs_listener_set_idle (self, g_value_get_boolean (value));
+        case PROP_SESSION_IDLE:
+                gs_listener_set_session_idle (self, g_value_get_boolean (value));
                 break;
         case PROP_THROTTLE_ENABLED:
                 gs_listener_set_throttle_enabled (self, g_value_get_boolean (value));
                 break;
+        case PROP_ACTIVATION_ENABLED:
+                gs_listener_set_activation_enabled (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -875,12 +1068,15 @@ gs_listener_get_property (GObject            *object,
         case PROP_ACTIVE:
                 g_value_set_boolean (value, self->priv->active);
                 break;
-        case PROP_IDLE:
-                g_value_set_boolean (value, self->priv->idle);
+        case PROP_SESSION_IDLE:
+                g_value_set_boolean (value, self->priv->session_idle);
                 break;
         case PROP_THROTTLE_ENABLED:
                 g_value_set_boolean (value, self->priv->throttle_enabled);
                 break;
+        case PROP_ACTIVATION_ENABLED:
+                g_value_set_boolean (value, self->priv->activation_enabled);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -976,6 +1172,14 @@ gs_listener_class_init (GSListenerClass *klass)
                                                                FALSE,
                                                                G_PARAM_READWRITE));
 
+        g_object_class_install_property (object_class,
+                                         PROP_ACTIVATION_ENABLED,
+                                         g_param_spec_boolean ("activation-enabled",
+                                                               NULL,
+                                                               NULL,
+                                                               TRUE,
+                                                               G_PARAM_READWRITE));
+
         g_type_class_add_private (klass, sizeof (GSListenerPrivate));
 }
 
@@ -990,8 +1194,9 @@ screensaver_is_running (DBusConnection *connection)
 
         dbus_error_init (&error);
         exists = dbus_bus_name_has_owner (connection, GS_LISTENER_SERVICE, &error);
-        if (dbus_error_is_set (&error))
+        if (dbus_error_is_set (&error)) {
                 dbus_error_free (&error);
+        }
 
         return exists;
 }
@@ -1047,12 +1252,13 @@ gs_listener_acquire (GSListener *listener,
         acquired = dbus_bus_request_name (listener->priv->connection,
                                           GS_LISTENER_SERVICE,
                                           0, &buserror) != -1;
-        if (dbus_error_is_set (&buserror))
+        if (dbus_error_is_set (&buserror)) {
                 g_set_error (error,
                              GS_LISTENER_ERROR,
                              GS_LISTENER_ERROR_ACQUISITION_FAILURE,
                              "%s",
                              buserror.message);
+        }
 
         dbus_error_free (&buserror);
 
@@ -1089,12 +1295,14 @@ gs_listener_finalize (GObject *object)
         g_return_if_fail (listener->priv != NULL);
 
 #if 0
-        if (listener->priv->connection)
+        if (listener->priv->connection) {
                 dbus_connection_unref (listener->priv->connection);
+        }
 #endif
 
-        if (listener->priv->inhibitors)
+        if (listener->priv->inhibitors) {
                 g_hash_table_destroy (listener->priv->inhibitors);
+        }
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
