@@ -1360,7 +1360,9 @@ get_user_on_host_name (void)
                 utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
         }
 
-        /* Translators: this is "username on hostname" */
+        /* Translators: this is "username on hostname".  You
+           can translate this to "%s@%s" if it is otherwise tricky
+           for your language. */
         str = g_strdup_printf (_("%s on %s"), utf8_name, host_name);
         g_free (utf8_name);
 
@@ -1450,6 +1452,12 @@ get_face_image ()
         return image;
 }
 
+#define INVISIBLE_CHAR_DEFAULT       '*'
+#define INVISIBLE_CHAR_BLACK_CIRCLE  0x25cf
+#define INVISIBLE_CHAR_WHITE_BULLET  0x25e6
+#define INVISIBLE_CHAR_BULLET        0x2022
+#define INVISIBLE_CHAR_NONE          0
+
 static void
 create_page_one (GSLockPlug *plug)
 {
@@ -1461,6 +1469,7 @@ create_page_one (GSLockPlug *plug)
         GtkWidget            *hbox;
         char                 *str;
         char                 *name;
+        gunichar              invisible_char;
 
         profile_start ("start", "page one");
 
@@ -1520,6 +1529,12 @@ create_page_one (GSLockPlug *plug)
         gtk_entry_set_activates_default (GTK_ENTRY (plug->priv->password_entry), TRUE);
         gtk_entry_set_visibility (GTK_ENTRY (plug->priv->password_entry), FALSE);
 
+        /* Only change the invisible character if it '*' otherwise assume it is OK */
+        if ('*' == gtk_entry_get_invisible_char (GTK_ENTRY (plug->priv->password_entry))) {
+                invisible_char = INVISIBLE_CHAR_BULLET;
+                gtk_entry_set_invisible_char (GTK_ENTRY (plug->priv->password_entry), invisible_char);
+        }
+
         gtk_label_set_mnemonic_widget (GTK_LABEL (password_label),
                                        plug->priv->password_entry);
 
